#include "cquad.h"

double cquad_dqawse(double f(), double a, double b, double alfa, double beta, int integr, double epsabs, double epsrel, double *abserr, int *neval, int *ier, double *alist, double *blist, double *rlist, double *elist, int *iord, int *last)
{
/******************************************************************************
***  Date written:         04/13/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Last revision date:   11/15/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Author:    Gianluca Russo    (gianlurusso@fastwebnet.it)               ***
***             Salvatore Cuomo   (salvatore.cuomo@dma.unina.it)            ***
***                                                                         ***
***             University of Naples "Federico II"                          ***
***             Dipartimento di Matematica e Applicazioni "R. Caccioppoli"  ***
***-------------------------------------------------------------------------***
***  Purpose:                                                               ***
***                                                                         ***
***    The function provides a C interface to the fortran routine dqawse    ***
***    of quadpack package.                                                 ***
***                                                                         ***
***    The function calculates an approximation result to a given definite  ***
***    integral i = integral of f(x) * w (x) over (a, b), where w shows a   ***
***    singular behaviour at the end points (see parameter integr),         *** 
***    hopefully satisfying following claim for accuracy:                   ***
***                                                                         ***
***              abs(i - result) <= max(epsabs, epsrel * abs(i))            ***
***-------------------------------------------------------------------------***
***  Description:                                                           ***
***                                                                         ***
***    Integrates f(x) * w(x) over (a, b) with a < b finite, and            ***
***    w(x) = ((x - a) * alfa) * ((b-x) * beta) * v(x) where:               ***
***                                                                         ***
***                        v(x) = 1 or                                      ***
***                        v(x) = LOG(x - a) or                             ***
***                        v(x) = LOG(b - x) or                             ***
***                        v(x) = LOG(x - a) * LOG(b - x)                   ***
***                                                                         ***
***    and alfa > (-1), beta > (-1).                                        ***
***                                                                         ***
***    The user specifies a, b, alfa, beta and the type of the function v.  ***
***    A globally adaptive subdivision strategy is applied, with modified   ***
***    Clenshaw-Curtis integration on the subintervals which contain a      ***
***    or b.                                                                ***
***                                                                         ***
***    Expert user version (provide more information and control than the   ***
***    easier version cquad_dqaws).                                         ***
***                                                                         ***
***    Double precision version.                                            ***
***-------------------------------------------------------------------------***
***  Input:                                                                 ***
***                                                                         ***
***    f        function subprogram defining the integrand function f(x)    ***
***                                                                         ***
***    a        lower limit of integration                                  ***
***                                                                         ***
***    b        upper limit of integration (b > a)                          ***
***                                                                         ***
***    alfa     parameter in the integrand function (alfa > -1)             ***
***                                                                         ***
***    beta     parameter in the integrand function (beta > -1)             ***
***                                                                         ***
***    integr   indicates which weight functions is to be used:             ***
***                                                                         ***
***             integr = 1 --> (x - a) * alfa * (b - x) * beta              ***
***             integr = 2 --> (x - a) * alfa * (b - x) * beta * log(x - a) ***
***             integr = 3 --> (x - a) * alfa * (b - x) * beta * log(b - x) ***
***             integr = 4 --> (x - a) * alfa * (b - x) * beta *            ***
***                            log(x - a) * log(b - x)                      ***
***                                                                         ***
***    epsabs   absolute accoracy requested                                 ***
***                                                                         ***
***    epsrel   relative accoracy requested                                 ***
***-------------------------------------------------------------------------***
***  Output:                                                                ***
***                                                                         ***
***    res      approximation to the integral                               ***
***                                                                         ***
***    abserr   estimate of the modulus of the absolute error               ***
***                                                                         ***
***    neval    number of integrand evaluations                             ***
***                                                                         ***
***    ier      normal termination flag:                                    ***
***                                                                         ***
***               ier = 0 --> normal and reliable termination of the        ***
***                           function. It is assumed that the requested    ***
***                           accuracy has been achieved.                   ***
***                                                                         ***
***               ier = 1 --> maximum number of subdivisions allowed has    ***
***                           been achieved.                                ***
***                                                                         ***
***               ier = 2 --> the occurrence of roundoff error is detected  ***
***                           which prevents the requested tolerance from   ***
***                           being achieved.                               ***
***                                                                         ***
***               ier = 3 --> extremely bad integrand behaviour occurs at   ***
***                           some points of the integration interval.      ***
***                                                                         ***
***               ier = 6 --> the input is invalid because b <= a, or       ***
***                           alfa <= -1, or beta <= -1, or integr < 1, or  ***
***                           integr > 4, or (epsabs <= 0 and epsrel < max( ***
***                           50 * rel.mach.acc., 0.5d-28)), or limit < 2.  ***
***                           In all these events res, abserr, neval,       ***
***                           rlist(1), elist(1), iord(1) and last are set  ***
***                           to zero.                                      ***
***                                                                         ***
***    alist    vector of dimension at least limit, the first last          ***
***             elements of which are the left end points of the            ***
***             subintervals in the partition of the given integration      ***
***             range (a,b)                                                 *** 
***                                                                         ***
***    blist    vector of dimension at least limit, the first last          ***
***             elements of which are the right end points of the           ***
***             subintervals in the partition of the given integration      ***
***             range (a,b)                                                 ***  
***                                                                         ***
***    rlist    vector of dimension at least limit, the first last          ***
***             elements of which are the integral approximations on the    ***
***             subintervals                                                ***
***                                                                         ***
***    elist    vector of dimension at least limit, the first last          ***
***             elements of which are the moduli of the absolute error      ***
***             estimates on the subintervals                               ***
***                                                                         ***
***    last     number of subintervals actually produced in the subdivision ***
***             process                                                     ***
***-------------------------------------------------------------------------***
***  Workspace:                                                             ***
***                                                                         ***
***    limit    gives an upper bound on the number of subintervals in the   ***
***             partition of (a,b) (at least 2)                             ***
***                                                                         ***
***    iord     vector of dimension at least limit, the first k elements    ***
***             of which are pointers to the error estimates over the       ***
***             subintervals, such that elist(iord(1)), ..., elist(iord(k)) ***
***             form a decreasing sequence, with k = last if last <=        ***
***             (limit/2 + 2), and k = limit + 1 - last otherwise           ***
***-------------------------------------------------------------------------***
***  References:      QUADPACK - Piessens, de Doncker, Uberhuber, Kahaner   ***
***                                                                         ***
***                   http://netlib.org                                     ***
******************************************************************************/

    int limit = LIMIT;
    double res;

    dqawse_(f, &a, &b, &alfa, &beta, &integr, &epsabs, &epsrel, &limit, &res, abserr, neval, ier, alist, blist, rlist, elist, iord, last);

    return res;
}
